@@ -4,6 +4,9 @@ import (
 	"bufio"
 	"errors"
 	"fmt"
+	"github.com/maaslalani/slides/internal/file"
+	"github.com/maaslalani/slides/internal/navigation"
+	"github.com/maaslalani/slides/internal/process"
 	"io"
 	"io/ioutil"
 	"os"
@@ -14,9 +17,7 @@ import (
 	tea "github.com/charmbracelet/bubbletea"
 	"github.com/charmbracelet/glamour"
 	"github.com/maaslalani/slides/internal/code"
-	"github.com/maaslalani/slides/internal/file"
 	"github.com/maaslalani/slides/internal/meta"
-	"github.com/maaslalani/slides/internal/process"
 	"github.com/maaslalani/slides/styles"
 )
 
@@ -33,6 +34,7 @@ type Model struct {
 	Paging   string
 	FileName string
 	viewport viewport.Model
+	buffer   string
 	// VirtualText is used for additional information that is not part of the
 	// original slides, it will be displayed on a slide and reset on page change
 	VirtualText string
@@ -99,29 +101,35 @@ func (m Model) Update(msg tea.Msg) (tea.Model, tea.Cmd) {
 		return m, nil
 
 	case tea.KeyMsg:
-		switch msg.String() {
-		case "ctrl+c", "q":
-			return m, tea.Quit
-		case " ", "down", "j", "right", "l", "enter", "n":
-			if m.Page < len(m.Slides)-1 {
-				m.Page++
-			}
-			m.VirtualText = ""
-		case "up", "k", "left", "h", "p":
-			if m.Page > 0 {
-				m.Page--
-			}
-			m.VirtualText = ""
+		keyPress := msg.String()
+
+		switch keyPress {
 		case "ctrl+e":
-			// Run code block
-			block, err := code.Parse(m.Slides[m.Page])
+			// Run code blocks
+			blocks, err := code.Parse(m.Slides[m.Page])
 			if err != nil {
 				// We couldn't parse the code block on the screen
 				m.VirtualText = "\n" + err.Error()
 				return m, nil
 			}
-			res := code.Execute(block)
-			m.VirtualText = res.Out
+			var outs []string
+			for _, block := range blocks {
+				res := code.Execute(block)
+				outs = append(outs, res.Out)
+			}
+			m.VirtualText = strings.Join(outs, "\n")
+		case "ctrl+c", "q":
+			return m, tea.Quit
+		default:
+			newState := navigation.Navigate(navigation.State{
+				Buffer:    m.buffer,
+				Page:     m.Page,
+				TotalSlides: len(m.Slides),
+			}, keyPress)
+			if newState.Page != m.Page {
+				m.VirtualText = ""
+			}
+			m.buffer, m.Page = newState.Buffer, newState.Page
 		}
 
 	case fileWatchMsg:
@@ -139,7 +147,7 @@ func (m Model) Update(msg tea.Msg) (tea.Model, tea.Cmd) {
 }
 
 func (m Model) View() string {
-	r, _ := glamour.NewTermRenderer(m.Theme, glamour.WithWordWrap(0))
+	r, _ := glamour.NewTermRenderer(m.Theme, glamour.WithWordWrap(m.viewport.Width))
 	slide := m.Slides[m.Page]
 	slide, err := r.Render(slide)
 	slide += m.VirtualText
